//-----------------------------------------------------------------------
// SPLITWND.CPP  This is the source for the module entrypoint, and
//               general main window manipulation functions, sizing,
//               painting, initialization. . .
//
// Written by George Cross, Borland C++ Developer Support, May 1996
// Copyright (c) 1991, 1996 by Borland International, All Rights Reserved
//-----------------------------------------------------------------------
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <strstrea.h>           //For ostrstream in AddDirectory() below
#include "splitwnd.h"
#include "resource.rh"

#define DIVIDER_WIDTH     2

static RECT g_rcClient;       // Save the client area so the controls
                              //   can remain proportionally sized
static RECT g_rcDivider;      // Position of the divider between
                              //   the controls
static LONG g_lPrevPos;       // Previous mouse position for resizing
static double g_dDividerProportion;   // Proportion of the screen that the
                                      //   is at divider
static HCURSOR g_hcurResize;          // Resizing cursor handle
       HCURSOR g_hcurWait;            // Hourglass cursor handle

HIMAGELIST  hImageList;

//-----------------------------------------------------------------------
// Dlg_OnNotify   Handles WM_NOTIFY messages
//-----------------------------------------------------------------------
bool Dlg_OnNotify(HWND /*hwnd*/, int /*id*/, LPNMHDR lpnmhdr)
{
  TV_INSERTSTRUCT  tvins;                        // TreeView Insert Struct.
  LPNM_TREEVIEW    lpnmTV;
  TV_ITEM          tvi;
  TCHAR            szFullPath[MAX_CHARS];

  switch (lpnmhdr->idFrom){
    case IDC_TREEVIEW1:
      lpnmTV = (LPNM_TREEVIEW)lpnmhdr;
      switch (lpnmhdr->code){
        case TVN_ITEMEXPANDING:

          //Fill TV_ITEM with info of the currently selected node
          //
          tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_PARAM |
                     TVIF_SELECTEDIMAGE;
          tvi.pszText = new TCHAR[MAX_CHARS];
          tvi.cchTextMax = MAX_CHARS;

          tvi.hItem = TreeView_GetSelection(lpnmhdr->hwndFrom);
          TreeView_GetItem(lpnmhdr->hwndFrom, &tvi);

          //If the node has never been expanded before, then add the
          //items of the underlying directory the tree below this node
          //
          if (tvi.lParam == NEVER_EXPANDED){

            //Indicate this node has already been expanded once.  This
            //prevents us from reading the directory redundantly
            //
            tvi.lParam = ALREADY_EXPANDED;
            TreeView_SetItem(lpnmhdr->hwndFrom, &tvi);

            //Add the contents of this directory as items of the tree
            //originating at this node
            //
            GetParentDirectory(lpnmhdr->hwndFrom, tvi.hItem, szFullPath);

            tvi.lParam         = NEVER_EXPANDED;        //Setting values
            tvins.item         = tvi;                   //of tvi struct
            tvins.hInsertAfter = TVI_LAST;              //for new nodes
            tvins.hParent      = lpnmTV->itemNew.hItem;

            AddDirectory(lpnmTV->hdr.hwndFrom,
                         szFullPath,
                         tvins);
          }
          delete[] tvi.pszText;              //clean up
          return false;                      //Allow node to expand
        default:
          break;
      }                                     //switch (lpnmTV->action)
      break;
    default:
      break;
  }                                         //switch (lpnmhdr->idFrom)
  return false;
}                                           //Dlg_OnNotify


void Dlg_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   HDC      hdc;

   hdc = BeginPaint(hwnd, &ps);

   DrawEdge(hdc, &g_rcDivider, EDGE_RAISED, NULL);

   EndPaint(hwnd, &ps);
}


void Dlg_OnSize (HWND hwnd, UINT /*state*/, int cx, int cy)
{
   // store the client area size for the next time we have to do this
   g_rcClient.right = cx - g_rcClient.left;
   g_rcClient.bottom = cy - g_rcClient.top;

   // set the new divider position to a proportional position
	g_rcDivider.left = (LONG) ((g_rcClient.right + g_rcClient.left)
                              * g_dDividerProportion);
   g_rcDivider.right = g_rcDivider.left + DIVIDER_WIDTH;
   g_rcDivider.bottom = g_rcClient.bottom;

   // Update the positions of the controls
   SetWindowPos(GetDlgItem(hwnd, IDC_TREEVIEW1),
               NULL,
               g_rcClient.left,
               g_rcClient.top,
               g_rcDivider.left - g_rcClient.left,
               cy,
               SWP_NOZORDER);

   SetWindowPos(GetDlgItem(hwnd, IDC_EDIT1),
               NULL,
               g_rcDivider.right,
               g_rcClient.top,
               g_rcClient.right - g_rcDivider.right,
               cy,
               SWP_NOZORDER);

   InvalidateRect(hwnd, &g_rcDivider, true);
   UpdateWindow(hwnd);
}


void Dlg_OnMouseMove(HWND hwnd, int x, int y, UINT /*keyFlags*/)
{
  if (GetCapture() != NULL)
  {
    DrawResizeLine(hwnd, g_lPrevPos);   // remove the previous line
    DrawResizeLine(hwnd, x);            // draw the new resizing line in
                                        // .. the current position
    g_lPrevPos = x;                     // save this position for next
  }                                     // .. time
  else
  {
    // if the mouse is over the resizing line, change the cursor
    POINT pt;

    pt.x = x; pt.y = y;
    if (PtInRect(&g_rcDivider, pt))
       SetCursor(g_hcurResize);
  }
}


void Dlg_OnLButtonDown(HWND hwnd, bool /*fDoubleClick*/,
                       int x, int y, UINT /*keyFlags*/)
{
  POINT pt;

  pt.x = x;
  pt.y = y;

  // Check to see if the mouse is in the divider
  if (PtInRect(&g_rcDivider, pt))
  {
    SetCursor(g_hcurResize);
    SetCapture(hwnd);             // keep track of the mouse until the
                                  // ..button is released;
    DrawResizeLine(hwnd, pt.x);   // draw the resizing rectangle
    g_lPrevPos = pt.x;            // save the position so the resize
                                  // ..rectangle can be moved
  }
}


void Dlg_OnLButtonUp(HWND hwnd, int x, int /*y*/, UINT /*keyFlags*/)
{
  if (GetCapture() != NULL)
  {
    SetCursor(g_hcurResize);
    // clean up from resize operation, release the capture and
    // remove the resizing line from the window
    ReleaseCapture();
    DrawResizeLine(hwnd, g_lPrevPos);

    // update the divider position
    g_rcDivider.left = x;
    g_rcDivider.right = g_rcDivider.left + DIVIDER_WIDTH;

    // Update the positions of the controls
    SetWindowPos(GetDlgItem(hwnd, IDC_TREEVIEW1),
               NULL,
               g_rcClient.left,
               g_rcClient.top,
               g_rcDivider.left - g_rcClient.left,
               g_rcClient.bottom - g_rcClient.top,
               SWP_NOZORDER);

    SetWindowPos(GetDlgItem(hwnd, IDC_EDIT1),
               NULL,
               g_rcDivider.right,
               g_rcClient.top,
               g_rcClient.right - g_rcDivider.right,
               g_rcClient.bottom - g_rcClient.top,
               SWP_NOZORDER);


    // find out where the divider was in proportion to the window size
    g_dDividerProportion = (double)(g_rcDivider.left) /
          (double)(g_rcClient.right + g_rcClient.left);

    // update the window
    InvalidateRect(hwnd, NULL, FALSE);
    UpdateWindow(hwnd);
  }
}


bool Dlg_OnInitDialog (HWND hwnd, HWND /*hwndFocus*/, LPARAM /*lParam*/)
{
  HWND hwndTV;
/*HWND hwndEB; */
  RECT rcClient;
  GetClientRect(hwnd, &rcClient);

/* hwndEB =*/CreateWindowEx(WS_EX_CLIENTEDGE,
                          "edit",
                          "\r\n\r\n\r\n    Editable Text . . .",
                          WS_CHILD | WS_VISIBLE | ES_MULTILINE |
                          ES_WANTRETURN | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
                          rcClient.left,
                          rcClient.top,
                          rcClient.right,
                          rcClient.bottom,
                          hwnd,
                          (HMENU)IDC_EDIT1,
                          GetModuleHandle(NULL),
                          NULL);

  hwndTV = CreateWindowEx(WS_EX_CLIENTEDGE,
                          WC_TREEVIEW,
                          "",
                          WS_VISIBLE | WS_CHILD | WS_BORDER |
                          TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT,
                          rcClient.left,
                          rcClient.top,
                          rcClient.right,
                          rcClient.bottom,
                          hwnd,
                          (HMENU)IDC_TREEVIEW1,
                          GetModuleHandle(NULL),
                          NULL);

  hImageList = CreateImageList();
  TreeView_SetImageList(hwndTV, hImageList, TVSIL_NORMAL);
  FillTreeView(hwndTV);


  // Set the initial size of the client area and divider position
  GetClientRect(hwnd, &g_rcClient);

  // The initial divider proportion is 50%
  g_dDividerProportion = 0.50;

  // The initial divider position is mid way down the client area
  g_rcDivider.left = (LONG) ((g_rcClient.right + g_rcClient.left)
                         * g_dDividerProportion);
  g_rcDivider.right = g_rcDivider.left + DIVIDER_WIDTH;
  g_rcDivider.top = g_rcClient.top;
  g_rcDivider.bottom = g_rcClient.bottom;

  // Call the size routine since Win95 and NT don't initally send
  // WM_SIZE to the window
  Dlg_OnSize(hwnd, 0, g_rcClient.right, g_rcClient.bottom);

  return true;
}


void Dlg_OnCommand (HWND hwnd, int id,
                    HWND /*hwndCtl*/, UINT /*codeNotify*/)
{
  switch (id){
    case IDCANCEL:
      EndDialog(hwnd,id);
      break;
  }
}


bool CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
                        WPARAM wParam, LPARAM lParam)
{
  bool fProcessed = true;
  switch (uMsg){
    HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
    HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);
    HANDLE_MSG(hDlg, WM_SIZE, Dlg_OnSize);
    HANDLE_MSG(hDlg, WM_PAINT, Dlg_OnPaint);
    HANDLE_MSG(hDlg, WM_LBUTTONDOWN, Dlg_OnLButtonDown);
    HANDLE_MSG(hDlg, WM_LBUTTONUP, Dlg_OnLButtonUp);
    HANDLE_MSG(hDlg, WM_MOUSEMOVE, Dlg_OnMouseMove);
    HANDLE_MSG(hDlg, WM_NOTIFY, Dlg_OnNotify);

    default:
      fProcessed = false;
      break;
  }
  return fProcessed;
}


void LoadCursors()
{
  // Resizing cursor
  g_hcurResize = LoadCursor(GetModuleHandle(NULL),
                            MAKEINTRESOURCE(IDC_RESIZE));
  if (!g_hcurResize)
     g_hcurResize = LoadCursor(NULL, MAKEINTRESOURCE(IDC_ARROW));

  // Hourglass
  g_hcurWait = LoadCursor(NULL,
                          MAKEINTRESOURCE(IDC_WAIT));
}


int APIENTRY WinMain (HINSTANCE hinstExe, HINSTANCE /*hinstPrev*/,
                      LPSTR /*lpszCmdLine*/, int /*nCmdShow*/)
{
  InitCommonControls();

  LoadCursors();

  DialogBox(hinstExe, MAKEINTRESOURCE(DIALOG_1),
            NULL, (DLGPROC)Dlg_Proc);
  return 0;
}
