//-----------------------------------------------------------------------
// TREEVIEW.CPP  This is the source for filling and maniuplating the
//               treeview control in the left pane of the splitter
//               splitter window.
//
// Written by George Cross, Borland C++ Developer Support, May 1996
// Copyright (c) 1991, 1996 by Borland International, All Rights Reserved
//-----------------------------------------------------------------------

#include <windows.h>
#include <commctrl.h>          //For the Win95 Tree View structures
#include <tchar.h>             //For Unicode
#include <strstrea.h>          //For ostrstream in AddDirectory() below
#include <shellapi.h>          //For SHFILEINFO in SetFileIcon() below
#include <checks.h>            //For diagnostics
#include "splitwnd.h"
#include "resource.rh"

extern HCURSOR g_hcurWait;     //Global handle to the hourglass cursor


//-----------------------------------------------------------------------
// CreateImageList   Sets up the bitmaps to be used in the Tree View
//                   window by creating an Image List.
//-----------------------------------------------------------------------
HIMAGELIST CreateImageList ()
{
/*
  HIMAGELIST hIL;       // Handle to ImageList
  HICON    hIcon;       // Handle to a icon bitmap
  static int        imageindex;

  // Create the Image List

  hIL = ImageList_Create(16 , // X size of one image
                         16 , // Y size of one image
                         ILC_MASK,   // Masked images
                         7,          // two images in this list
                         0);         // No "growing" bitmaps in this list.

  if (!hIL) return NULL;

  // Add Each bitmap to the ImageList.
  //
  // ImageList_AddMasked will add the bitmap, and treat every
  // pixel that is (in this example) white as a "transparent" pixel,
  // since we specified TRUE for fMask in the above call to
  // ImageList_Create.

  for (int i = IDI_CD; i <= IDI_APP; i++){

    hIcon = LoadIcon(GetModuleHandle(NULL),
                     MAKEINTRESOURCE(i));
    if (hIcon){
        imageindex = ImageList_ReplaceIcon(hIL, -1, hIcon);
    }                                       //End if
  }                                         //End for

  return hIL;
*/
  SHFILEINFO sfi;
  HIMAGELIST
  hSysImageList = (HIMAGELIST) SHGetFileInfo(__TEXT("c:\\"),
                                0,
                                &sfi,
                                sizeof(SHFILEINFO),
                                SHGFI_SMALLICON | SHGFI_SYSICONINDEX);
  return hSysImageList;

}


//
// Recursively traverse the directory hierarchy of the given dirPath.
// dirPath is specified by this format:
//   [drive]:\\{dirs}\\*.*
// where drive is a letter, typically c through z and
//       dirs is zero or more directories separated by a backslash.
//
void  AddDirectory(HWND hWndTreeView, LPCTSTR dirPath,
                   TV_INSERTSTRUCT tvins)
{
  WIN32_FIND_DATA  FindFile;                     // For file searches.
  HANDLE           hFindFile;                    // For file searches.
  HCURSOR          hcurOld;
  TCHAR            pszFullFileName[MAX_CHARS];

  //Copy the directory path to a new string and append the new
  //string with the wildcard character
  _tcscpy(pszFullFileName, dirPath);
  _tcscat(pszFullFileName, __TEXT("*.*"));

  //Turn on wait cursor
  hcurOld = SetCursor(g_hcurWait);

  //Open a file handle on this directory for searching.
  //
  hFindFile = FindFirstFile(pszFullFileName, &FindFile);
  if (INVALID_HANDLE_VALUE != hFindFile) {

    //Add all items in this directory to the Tree View on the node
    //passed in the parameter 'parent'
    //
    do{
      //Don't bother displaying the ..\ or .\ directories (current
      //and parent)
      //
      if (FindFile.cFileName[0] == '.') continue;

      //Assign file name
      tvins.item.pszText = FindFile.cFileName;

      //If this item of the directory is itself another directory,
      //then indicate it this has children so that a '+' appears beside it
      //Otherwise make sure a '+' does NOT appear beside it
      //
      if (FindFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY){
        tvins.item.mask |= TVIF_CHILDREN;

/*
        tvins.item.iImage = IDI_CLOSEDIR - IDI_CD;
        tvins.item.iSelectedImage = IDI_OPENDIR - IDI_CD;
*/
      }
      else{
        tvins.item.mask &= ~TVIF_CHILDREN;
//        SetFileIcon(hWndTreeView, &tvins, dirPath);
      }

      _tcscpy(pszFullFileName, dirPath);
      _tcscat(pszFullFileName, FindFile.cFileName);
      SetFileIcon(hWndTreeView, &tvins, pszFullFileName);

      //Add item to the tree
      //
      TreeView_InsertItem(hWndTreeView, &tvins);

      //If this item of the directory is itself another directory,
      //then recurse this function "AddDirectory" in order to add
      //this item's directory information to the Tree View
      //
/*
      if (FindFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY){
        ostrstream os1;
        os1 << dirPath << "\\" << FindFile.cAlternateFileName << ends;

        //Save the current handle to the parent item and set the new one
        HTREEITEM htemp = tvins.hParent;
        tvins.hParent = hPrev;

        AddDirectory(hWndTreeView, os1.str(), tvins);
        delete[] os1.str();                  //Restore things . . .
        tvins.hParent = htemp;
      }                                      //if (FindFile.dw . . .
*/
    }while(FindNextFile(hFindFile, &FindFile));
    FindClose(hFindFile);
  }                                          //if (INVALID_HANDLE_VALUE

  //Turn off wait cursor
  SetCursor(hcurOld);
}                                            //AddDirectory()


/////////////////////////////////////////////////////////////////////////
// FillTreeView
// This function retrieves all the logical drives and inserts icons
// and text for each one into the tree view.
/////////////////////////////////////////////////////////////////////////
void FillTreeView(HWND        hwndTreeView)
{
  TV_ITEM          tvi;                          // TreeView Item.
  TV_INSERTSTRUCT  tvins;                        // TreeView Insert Struct.

  //Initialize the Tree View item structure to use the folder image
  //
  tvi.mask            = TVIF_TEXT | TVIF_IMAGE | TVIF_PARAM |
                        TVIF_SELECTEDIMAGE | TVIF_CHILDREN;
  tvi.iImage          = IDI_CD - IDI_CD;
  tvi.iSelectedImage  = IDI_CD - IDI_CD;
  tvi.cchTextMax      = MAX_CHARS;
  tvi.lParam          = NEVER_EXPANDED;

  //Insert items at root level since they are the logical drives.
  //
  tvins.item         = tvi;
  tvins.hInsertAfter = TVI_LAST;
  tvins.hParent      = TVI_ROOT;

  //Retrieve the list of logical drives for this system
  //
  TCHAR drivesToAdd[MAX_DRIVELEN];
  DWORD rcode = GetLogicalDriveStrings(MAX_DRIVELEN, drivesToAdd);
  if (rcode <= MAX_DRIVELEN) {

    tvins.item.pszText = drivesToAdd;

    //Traverse string of logical drives adding each one to the tree
    //
    do {

      //Adjust the icon according to the drive type
       SetFileIcon(hwndTreeView, &tvins, tvins.item.pszText);

      //Remove the trailing backslash
      int length = _tcslen(tvins.item.pszText) + sizeof(TCHAR);
      if (tvins.item.pszText[_tcslen(tvins.item.pszText) - sizeof(TCHAR)]
          == '\\')
        tvins.item.pszText[_tcslen(tvins.item.pszText) - sizeof(TCHAR)]
          = '\0';

      //Add the drive to the tree
      TreeView_InsertItem(hwndTreeView, &tvins);

      //Advance the string pointer through the list of logical drives
      tvins.item.pszText += length;

    } while(tvins.item.pszText[0] != '\0');
  }
}


//
// Retrieves the directory of the selected item in the TTreeWindow.
//
void GetParentDirectory(HWND hWnd, HTREEITEM treeNode, LPTSTR path)
{
  path[0] = '\0';
  do{
    TCHAR toBeAdded[MAX_CHARS];
    _tcscpy(toBeAdded, path);
    LPTSTR buffer = new TCHAR[MAX_CHARS];
    buffer[0] = '\0';

    TV_ITEM tvi;
    tvi.mask = TVIF_TEXT;
    tvi.pszText = buffer;
    tvi.cchTextMax = MAX_CHARS;
    tvi.hItem = treeNode;
    TreeView_GetItem(hWnd, &tvi);

    _tcscpy(path, buffer);
    _tcscat(path, "\\");
    _tcscat(path, toBeAdded);

    delete[] buffer;
    treeNode = TreeView_GetParent(hWnd, treeNode);
  } while (treeNode != NULL);
}


/////////////////////////////////////////////////////////////////////////
// SetDriveIcon
// Just sets the icon to be used to represent the drive type.
/////////////////////////////////////////////////////////////////////////
void SetDriveIcon(TV_INSERTSTRUCT* ptvins)
{
  //Turn off warning that the floppy drive contains no disk
  UINT oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);

  //Set icon accordingly
  switch (GetDriveType(ptvins->item.pszText)){
    case DRIVE_CDROM:
      ptvins->item.iImage = IDI_CD - IDI_CD;
      ptvins->item.iSelectedImage = IDI_CD - IDI_CD;
      break;
    case DRIVE_FIXED:
      ptvins->item.iImage = IDI_FIXEDDRV - IDI_CD;
      ptvins->item.iSelectedImage = IDI_FIXEDDRV - IDI_CD;
      break;
    case DRIVE_REMOTE:
      ptvins->item.iImage = IDI_NETDRIVE - IDI_CD;
      ptvins->item.iSelectedImage = IDI_NETDRIVE - IDI_CD;
      break;
    case DRIVE_REMOVABLE:
      ptvins->item.iImage = IDI_FLOPPY - IDI_CD;
      ptvins->item.iSelectedImage = IDI_FLOPPY - IDI_CD;
      break;
    default:
      break;
  }

  //Reset error mode
  SetErrorMode(oldErrorMode);

}


/////////////////////////////////////////////////////////////////////////
// SetFileIcon
// Just sets the icon to be used to represent the file type.
/////////////////////////////////////////////////////////////////////////
void SetFileIcon(HWND /*hWndTreeView*/, TV_INSERTSTRUCT* ptvins,
                 LPCTSTR pzfileName)
{
//  HIMAGELIST hSysImageList;
//  HIMAGELIST hOldImageList;
//  HICON      hIcon;
  SHFILEINFO sfi;

  /*hSysImageList = (HIMAGELIST)*/ SHGetFileInfo(pzfileName,
                                0,
                                &sfi,
                                sizeof(SHFILEINFO),
                                SHGFI_SMALLICON |
                                SHGFI_SYSICONINDEX);


  /*
   hIcon = ImageList_GetIcon (hSysImageList,
                              sfi.iIcon,
                              ILD_NORMAL);

   hOldImageList  = TreeView_GetImageList(hWndTreeView, TVSIL_NORMAL);

   ImageList_ReplaceIcon(hOldImageList, 6, hIcon);


  hOldImageList = TreeView_SetImageList(hWndTreeView,
                                        hSysImageList,
                                        TVSIL_NORMAL);
 */
  ptvins->item.iImage = sfi.iIcon;
  ptvins->item.iSelectedImage = sfi.iIcon;

 // TreeView_SetImageList(hWndTreeView, hOldImageList, TVSIL_NORMAL);
}



